/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung - Bibliothek
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import java.util.ArrayList;
import java.util.List;

/**
 * Repräsentiert eine Attributliste. Der Attributtyp ist hier wieder Attribut.
 *
 * @author BitCtrl Systems GmbH, Schumann, Peuker
 */
class Attributliste extends AbstractAttribut<List<Attribut<?>>> {

	/**
	 * Alle Objekte vom Typ <code>Attributliste</code> mit der gleichen UID
	 * können paarweise serialisiert und deserialisiert werden. Objekte mit
	 * unterschiedlicher UID sind inkompatibel.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * die Liste der Attribute.
	 */
	private List<Attribut<?>> liste = new ArrayList<>();

	/**
	 * konstruiert eine leere Attributliste.
	 *
	 * @param name
	 *            Name der Attributliste
	 */
	Attributliste(final String name) {
		super(name);
	}

	/**
	 * konstruiert eine Attributliste.
	 *
	 * @param name
	 *            Attributname
	 * @param liste
	 *            Eine Liste von Attributen
	 */
	Attributliste(final String name, final List<Attribut<?>> liste) {
		super(name);
		this.liste = liste;
	}

	/**
	 * fügt der Attributliste ein neues Attribut hinzu.
	 *
	 * @param attribut
	 *            Ein neues Attribut
	 * @return <code>true</code>, wenn das Attribut hinzugefügt wurde;
	 *         <code>false</code>, wenn bereits ein Attribut mit dem gleichem
	 *         Namen in der Attributliste existiert
	 */
	boolean add(final Attribut<?> attribut) {
		return liste.add(attribut);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion liefert die Liste der Attribute, aus denen die Attributliste
	 * besteht zurück.
	 */
	@Override
	public List<Attribut<?>> get() {
		return liste;
	}

	/**
	 * gibt das Attribut mit dem angegebenen Namen zurück. Wirft eine
	 * {@link java.lang.IllegalArgumentException}, wenn kein Attribut mit dem
	 * angegebenen Namen existiert.
	 *
	 * @param attributName
	 *            Name des gesuchten Attributs
	 * @return Das Attribut
	 */
	Attribut<?> getAttribut(final String attributName) {
		for (final Attribut<?> att : liste) {
			if (att.getName().equals(attributName)) {
				return att;
			}
		}

		throw new IllegalArgumentException("Es existiert kein Attribut mit diesem Namen.");
	}

	/**
	 * prüft ob ein bestimmtes Attribut in der Attributgruppe vorhanden ist.
	 *
	 * @param attributName
	 *            Name des gesuchten Attributs
	 * @return <code>true</code>, wenn das Attribut existiert, sonst
	 *         <code>false</code>
	 */
	boolean hasAttribut(final String attributName) {
		for (final Attribut<?> att : liste) {
			if (att.getName().equals(attributName)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * {@inheritDoc}. Die Funktion entfernt alle in der Attributliste
	 * gespeicherten Attribute und fügt die übergebenen wieder ein.
	 */
	@Override
	public void set(final List<Attribut<?>> wert) {
		liste.clear();
		liste.addAll(wert);
	}

}
